'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _d_js = require('d_js');

var _d_js2 = _interopRequireDefault(_d_js);

var _intersectionObserver = require('intersection-observer');

var _intersectionObserver2 = _interopRequireDefault(_intersectionObserver);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var PageLoader = function () {
    function PageLoader(resultSelector, buttonSelector) {
        var _this = this;

        var context = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : document;

        _classCallCheck(this, PageLoader);

        this.events = {};

        this.resultSelector = resultSelector;
        this.result = context.querySelector(resultSelector);

        this.buttonSelector = buttonSelector;
        this.button = context.querySelector(buttonSelector);

        this.currentPage = {
            title: document.title,
            url: document.location.href,
            target: null
        };

        this.pages = [this.currentPage];

        if (this.button) {
            _d_js2.default.on('click', this.button, onClick.bind(this));
        }

        this.observer = new IntersectionObserver(function (entries) {
            entries.forEach(function (entry) {
                var page = _this.pages.find(function (page) {
                    return page.target === entry.target;
                });

                if (page) {
                    if (isNextPage(entry)) {
                        _this.changePage(page);
                    } else if (isPreviousPage(entry)) {
                        _this.changePage(page.previous);
                    }
                }
            });
        });
    }

    _createClass(PageLoader, [{
        key: 'on',
        value: function on(event, callback) {
            if (!this.events[event]) {
                this.events[event] = [];
            }

            this.events[event].push(callback);
        }
    }, {
        key: 'off',
        value: function off(event, callback) {
            if (this.events[event]) {
                if (!callback) {
                    delete this.events[event];
                } else {
                    var index = this.events[event].indexOf(callback);

                    if (index !== -1) {
                        this.events[event].splice(index, 1);
                    }
                }
            }
        }
    }, {
        key: 'trigger',
        value: function trigger(event) {
            var _this2 = this;

            var args = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

            if (this.events[event]) {
                this.events[event].forEach(function (callback) {
                    return callback.apply(_this2, args);
                });
            }
        }
    }, {
        key: 'loadPage',
        value: function loadPage(url) {
            var _this3 = this;

            this.trigger('beforeLoadPage', [url]);

            return fetch(url).then(function (response) {
                return response.text();
            }).then(function (html) {
                var doc = document.implementation.createHTMLDocument();
                doc.documentElement.innerHTML = html;

                var result = doc.documentElement.querySelector(_this3.resultSelector);
                var button = doc.documentElement.querySelector(_this3.buttonSelector);
                var page = {
                    title: doc.title,
                    url: url,
                    target: result.firstElementChild,
                    previous: _this3.pages[_this3.pages.length - 1]
                };

                //if the button is the last element of the result
                var last = _this3.result.lastElementChild;
                if (last === _this3.button || last.contains(_this3.button)) {
                    toArray(result.children).filter(function (child) {
                        return child !== button && !child.contains(button);
                    }).forEach(function (child) {
                        return last.before(child);
                    });
                } else {
                    toArray(result.children).forEach(function (child) {
                        return _this3.result.append(child);
                    });
                }

                _this3.pages.push(page);
                _this3.observer.observe(page.target);
                _this3.trigger('loadPage', [page]);
                _this3.changePage(page, true);

                if (button) {
                    _this3.button.replaceWith(button);

                    _d_js2.default.on('click', button, onClick.bind(_this3));
                } else {
                    _this3.button.remove();
                }

                _this3.button = button;
            });
        }
    }, {
        key: 'changePage',
        value: function changePage(page) {
            var isNew = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

            if (page !== this.currentPage) {
                if (isNew) {
                    history.pushState({}, page.title, page.url);
                } else {
                    history.replaceState({}, page.title, page.url);
                }

                document.title = page.title;
                this.currentPage = page;
                this.trigger('changePage', [page]);
            }
        }
    }]);

    return PageLoader;
}();

exports.default = PageLoader;


function onClick(event) {
    var url = this.button.getAttribute('href');

    if (!url) {
        return;
    }

    event.preventDefault();

    this.loadPage(url).catch(function (err) {
        console.error(err);
        document.location = url;
    });
}

function isNextPage(entry) {
    return entry.isIntersecting && entry.boundingClientRect.top > 0;
}

function isPreviousPage(entry) {
    return !entry.isIntersecting && entry.boundingClientRect.top > 0;
}

function toArray(el) {
    return Array.prototype.slice.call(el);
}